﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using C1.C1Excel;
using System.Globalization;

namespace Wf_C1Excel_AutoSizeCols
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            this.button1.Click += new EventHandler(button1_Click);
        }

        void button1_Click(object sender, EventArgs e)
        {
            c1XLBook1.Clear();

            /*Create test Styles*/
            XLStyle s1 = new XLStyle(c1XLBook1);
            XLStyle s2 = new XLStyle(c1XLBook1);
            XLStyle s3 = new XLStyle(c1XLBook1);
            s1.Format = "#,##0.00000";
            s2.Format = "#,##0.00000";
            /*Assign a large enough Font*/
            s2.Font = new Font("Courier New", 14);
            s3.Format = "dd-MMM-yy";

            C1.C1Excel.XLSheet sheet = c1XLBook1.Sheets[0];
            Random r = new Random();
            for (int i = 0; i < 100; i++)
            {
                sheet[i, 0].Value = r.NextDouble() * 100000;
                /*Assign the style with enlarged Font to every cell in 13th Row of first Column*/
                sheet[i, 0].Style = (i % 13 == 0) ? s2 : s1;
            }
            for (int i = 0; i < 100; i++)
            {
                sheet[i, 1].Value = new DateTime(2005, r.Next(1, 12), r.Next(1, 28));
                sheet[i, 1].Style = s3;
            }

            string tempdir = Application.ExecutablePath.Substring(0,
                Application.ExecutablePath.LastIndexOf("\\") + 1);

            /*Save sheet with default column width*/
            c1XLBook1.Save(tempdir + @"defaultWidth.xls");

            /*Resize columns*/
            AutoSizeColumns(sheet);

            /*Save sheet with resized columns*/
            c1XLBook1.Save(tempdir + @"autoSized.xls");

            System.Diagnostics.Process.Start(tempdir + @"autoSized.xls");            
        }

        private void AutoSizeColumns(XLSheet sheet)
        {
            /*No Graphics instance available because there's no Paint event*/
            /*Create a Graphics object using a handle to current window instead*/
            using (Graphics g = Graphics.FromHwnd(IntPtr.Zero))
            {
                /*Traverse rows and columns*/
                for (int c = 0; c < sheet.Columns.Count; c++)
                {
                    int colWidth = -1;
                    for (int r = 0; r < sheet.Rows.Count; r++)
                    {
                        /*Get cell value*/
                        object value = sheet[r, c].Value;
                        if (value != null)
                        {
                            string text = value.ToString();

                            /*Get Style for this cell*/
                            C1.C1Excel.XLStyle s = sheet[r, c].Style;
                            if (s != null && s.Format.Length > 0 && value is IFormattable)
                            {
                                string fmt = XLStyle.FormatXLToDotNet(s.Format);
                                /*get formatted text*/
                                text = ((IFormattable)value).ToString(fmt, CultureInfo.CurrentCulture);
                            }

                            Font font = this.c1XLBook1.DefaultFont;
                            if (s != null && s.Font != null)
                            {
                                font = s.Font;
                            }

                            /*Get size of drawn string according to its Font*/
                            Size sz = Size.Ceiling(g.MeasureString(text + "XX", font));
                            
                            if (sz.Width > colWidth)
                                colWidth = sz.Width;
                        }
                    }
                    /*Set columns width*/
                    if (colWidth > -1)
                        sheet.Columns[c].Width = C1XLBook.PixelsToTwips(colWidth);
                }
            }

        }
    }
}
